{
XPMenu for Delphi
Author: Khaled Shagrouni
URL: http://www.shagrouni.com
e-mail: shagrouni@hotmail.com
Version 1.0, 19 June, 2001


XPMenu is a Delphi component to mimic Office XP menu style.
Copyright (C) 2001 Khaled Shagrouni.

This component is FREEWARE with source code. I still hold the copyright.
If you make any modifications to the code, please send them to me.
If have any ideas for improvement or bug reports, don't hesitate to e-mail me.
If you wish to be informed about any enhancement, email me with
subject titled "XPMenu" (you can leave the message body empty).


Short Description:
=================
A none visual component that changes the visual aspects of
the menu/pop-up menu in the host form to mimic Office XP style.

Usage:
======
- Place TXPMenu in a form, set Active property to true. The new style should
  take effect in design mode.

- Make sure that the creation order of TXPMenu comes after any menu component.
  To change the creation order, choose Edit | Creation Order from Delphi menu
  to open the Creation Order dialog box.

- To force the XPmenu to include menu items created in run time,
  reset Active property to true (no need to set it to false first).

- If you want to target a different form other than the one hosting the
  component; set Form property (public) to that form.

History:
========
This component is based on code which I have posted at Delphi3000.com
(http://www.delphi3000/articles/article_2246.asp) and Borland Code-Central
(http://codecentral.borland.com/codecentral/ccweb.exe/listing?id=16120).

Future releases:
================
Well, this is the first version of the component, I expect a lot of suggestions
for enhancement or bug fixes, I also plan to add more features. So please
tune in and check the latest at my site www.shagrouni.com .


}
unit XPMenu;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, Forms, Menus;

type
  TXPMenu = class(TComponent)
  private
    FActive: boolean;
    FForm: TForm;
    FFont: TFont;
    FColor: TColor;
    FIconBackColor: TColor;
    FMenuBarColor: TColor;
    FCheckedColor: TColor;
    FSeparatorColor: TColor;
    FSelecBorderColor: TColor;
    FSelectColor: TColor;
    FDisabledColor: TColor;
    FSelectFontColor: TColor;
    FIconWidth: integer;
    FDrawSelect: boolean;


    procedure SetActive(const Value: boolean);
    procedure SetForm(const Value: TForm);
    procedure SetFont(const Value: TFont);
    procedure SetColor(const Value: TColor);
    procedure SetIconBackColor(const Value: TColor);
    procedure SetMenuBarColor(const Value: TColor);
    procedure SetCheckedColor(const Value: TColor);
    procedure SetDisabledColor(const Value: TColor);
    procedure SetSelectColor(const Value: TColor);
    procedure SetSelecBorderColor(const Value: TColor);
    procedure SetSeparatorColor(const Value: TColor);
    procedure SetSelectFontColor(const Value: TColor);
    procedure SetIconWidth(const Value: integer);
    procedure SetDrawSelect(const Value: boolean);


  protected
    procedure InitMenueItems(Enable: boolean);
    procedure DrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect;
      Selected: Boolean);
    procedure MenueDrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect;
      Selected: Boolean);
    procedure MeasureItem(Sender: TObject; ACanvas: TCanvas;
      var Width, Height: Integer);
    function GetImageExtent(MenuItem: TMenuItem): TPoint;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Form: TForm read FForm write SetForm;
  published
    property Active: boolean read FActive write SetActive;
    property Font: TFont read FFont write SetFont;
    property Color: TColor read FColor write SetColor;
    property IconBackColor: TColor read FIconBackColor write SetIconBackColor;
    property MenuBarColor: TColor read FMenuBarColor write SetMenuBarColor;
    property SelectColor: TColor read FSelectColor write SetSelectColor;
    property SelecBorderColor: TColor read FSelecBorderColor
      write SetSelecBorderColor;
    property SelectFontColor: TColor read FSelectFontColor
      write SetSelectFontColor;
    property DisabledColor: TColor read FDisabledColor write SetDisabledColor;
    property SeparatorColor: TColor read FSeparatorColor write SetSeparatorColor;
    property CheckedColor: TColor read FCheckedColor write SetCheckedColor;
    property IconWidth: integer read FIconWidth write SetIconWidth;
    property DrawSelect: boolean read FDrawSelect write SetDrawSelect;

  end;

  function GetShadeColor(clr:TColor; Value: byte): TColor;
procedure Register;

implementation
{$R XPMenu.dcr}

procedure Register;
begin
  RegisterComponents('XP', [TXPMenu]);
end;

{ TXPMenue }

constructor TXPMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFont := TFont.Create;
  FFont.Name := 'Tahoma';
  FForm := Tform(Owner);

  FColor := $00F7F8F9;
  FIconBackColor := $00D1D8DB;
  FMenuBarColor := FForm.Color;
  FSelectColor := $00D4B8B3;
  FSelecBorderColor := $006A240A;

  FDisabledColor := clGray;
  FSeparatorColor := $00ACACAD;
  FCheckedColor := $00505050;
  FIconWidth := 24;
  FDrawSelect := true;
  if FActive then
    begin
      InitMenueItems(true);
    end;
end;

destructor TXPMenu.Destroy;
begin
  InitMenueItems(false);
  FFont.Free;

  inherited;
end;


procedure TXPMenu.InitMenueItems(Enable: boolean);
var
  i: integer;
  FMenuItem: TMenuItem;
begin
  for i := 0 to FForm.ComponentCount - 1 do
    begin
      if FForm.Components[i] is TMenuItem then
        begin
          FMenuItem := TMenuItem(FForm.Components[i]);

          if Enable then
            begin
              if not assigned(FMenuItem.OnDrawItem) then
                FMenuItem.OnDrawItem := DrawItem;
              if not assigned(FMenuItem.OnMeasureItem) then
                FMenuItem.OnMeasureItem := MeasureItem;
            end
          else
            begin
              if addr(FMenuItem.OnDrawItem) = addr(TXPMenu.DrawItem) then
                FMenuItem.OnDrawItem := nil;
              if addr(FMenuItem.OnMeasureItem) = addr(TXPMenu.MeasureItem) then
                FMenuItem.OnMeasureItem := nil;
            end;
        end;
        if FForm.Components[i] is TPopupMenu then
          TPopupMenu(FForm.Components[i]).OwnerDraw := true;

        if FForm.Components[i] is TMainMenu then
          TMainMenu(FForm.Components[i]).OwnerDraw := true;
      end;


end;

procedure TXPMenu.DrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect;
  Selected: Boolean);
begin
  if FActive then
    MenueDrawItem(Sender, ACanvas, ARect, Selected);
end;



function TXPMenu.GetImageExtent(MenuItem: TMenuItem): TPoint;
var
  HasImgLstBitmap: boolean;
  B: TBitmap;
begin
  B := TBitmap.Create;
  B.Width := 0;
  B.Height := 0;
  Result.x := 0;
  Result.Y := 0;
  HasImgLstBitmap := false;

  if FForm.Menu <> nil then
  if MenuItem.GetParentComponent.Name = FForm.Menu.Name then
    begin
      if FForm.Menu.Images <> nil then
        if MenuItem.ImageIndex <> -1 then
          HasImgLstBitmap := true ;

    end;

  if (MenuItem.Parent.GetParentMenu.Images <>  nil) or
     (MenuItem.Parent.SubMenuImages <> nil)  then
    begin
      if MenuItem.ImageIndex <> -1 then
        HasImgLstBitmap := true
      else
        HasImgLstBitmap := false;
    end;

  if HasImgLstBitmap then
    begin

      if MenuItem.Parent.SubMenuImages <> nil then
        MenuItem.Parent.SubMenuImages.GetBitmap(MenuItem.ImageIndex, B)
      else
        MenuItem.Parent.GetParentMenu.Images.GetBitmap(MenuItem.ImageIndex, B)
    end
  else
    if MenuItem.Bitmap.Width > 0 then
      B.Assign(TBitmap(MenuItem.Bitmap));

  Result.x := B.Width;
  Result.Y := B.Height;

  B.Free;
end;

procedure TXPMenu.MeasureItem(Sender: TObject; ACanvas: TCanvas;
  var Width, Height: Integer);
var
  s: string;
  W,H: integer;
  P: TPoint;
begin
  if FActive then
    begin
      S := TMenuItem(Sender).Caption;
      if TMenuItem(Sender).IsLine then
         S := '';
      if Trim(ShortCutToText(TMenuItem(Sender).ShortCut)) <> '' then
        S := S +  ShortCutToText(TMenuItem(Sender).ShortCut) + 'W';

      ACanvas.Font.Assign(FFont);
      W := ACanvas.TextWidth(s) ;
      if pos('&', s) > 0 then
        W := W - ACanvas.TextWidth('&') ;

      P := GetImageExtent(TMenuItem(Sender));

      W := W + P.x + 10;
      if Width < W then
        Width := W;

      if TMenuItem(Sender).IsLine then
        Height := 4
      else
        begin
          H := ACanvas.TextHeight(s) + Round(ACanvas.TextHeight(s)* 0.75);
          if P.y + 4 > H then
            H :=  P.y + 4;

          if Height < H then
            Height := H;
        end;
    end;

end;

procedure TXPMenu.MenueDrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect;
  Selected: Boolean);
var
  txt: string;
  B: TBitmap;

  IConRect, TextRect, CheckedRect: TRect;

  DefColor, HoldColor: TColor;
  i, X1, X2: integer;
  TextFormat: integer;
  HasImgLstBitmap: boolean;
  FMenuItem: TMenuItem;
  FMenu: TMenu;
  FTopMenu: boolean;

begin
  FTopMenu := false;
  FMenuItem := TMenuItem(Sender);
  FMenu := FMenuItem.Parent.GetParentMenu;
  DefColor := FFont.Color;
  ACanvas.Font.Assign (FFont);
  if FMenu.IsRightToLeft then
    ACanvas.Font.Charset := ARABIC_CHARSET;

  Inc(ARect.Bottom, 1);
  TextRect := ARect;
  txt := ' ' + FMenuItem.Caption;

  B := TBitmap.Create;

  B.Transparent := True;
  B.TransparentMode := tmAuto;

  HasImgLstBitmap := false;

  if (FMenuItem.Parent.GetParentMenu.Images <>  nil) or
     (FMenuItem.Parent.SubMenuImages <> nil) then
    begin
      if FMenuItem.ImageIndex <> -1 then
        HasImgLstBitmap := true
      else
        HasImgLstBitmap := false;
    end;


  if HasImgLstBitmap then
    begin
      if FMenuItem.Parent.SubMenuImages <> nil then
        FMenuItem.Parent.SubMenuImages.GetBitmap(FMenuItem.ImageIndex, B)
      else
        FMenuItem.Parent.GetParentMenu.Images.GetBitmap(FMenuItem.ImageIndex, B)
    end
  else
    if FMenuItem.Bitmap.Width > 0 then
      B.Assign(TBitmap(FMenuItem.Bitmap))
    else
      HasImgLstBitmap := false;


  if FMenu.IsRightToLeft then
    begin
      X1 := ARect.Right - FIconWidth;
      X2 := ARect.Right;
    end
  else
    begin
      X1 := ARect.Left;
      X2 := ARect.Left + FIconWidth;
    end;
  IConRect := Rect(X1, ARect.Top, X2, ARect.Bottom);


  if HasImgLstBitmap then
    begin
      CheckedRect := IconRect;
      Inc(CheckedRect.Left, 2);
      Inc(CheckedRect.Top, 2);
      Dec(CheckedRect.Right, 2);
      Dec(CheckedRect.Bottom, 2);

    end
  else
    begin
      CheckedRect.Left := IConRect.Left +
                          (IConRect.Right - IConRect.Left - 10) div 2;
      CheckedRect.Top := IConRect.Top +
                         (IConRect.Bottom - IConRect.Top - 10) div 2;
      CheckedRect.Right := CheckedRect.Left + 10;
      CheckedRect.Bottom := CheckedRect.Top + 10;

    end;


  if FMenu.IsRightToLeft then
    begin
      X1 := ARect.Left;
      X2 := ARect.Right - FIconWidth;
      if B.Width > FIconWidth then
        X2 := ARect.Right - B.Width - 4;
    end
  else
    begin
      X1 := ARect.Left + FIconWidth;
      if B.Width > X1 then
        X1 := B.Width + 4;
      X2 := ARect.Right;
    end;

  TextRect := Rect(X1, ARect.Top, X2, ARect.Bottom);

  if FMenu is TMainMenu then
    for i := 0 to FMenuItem.GetParentMenu.Items.Count - 1 do
      if FMenuItem.GetParentMenu.Items[i] = FMenuItem then
        begin
          FTopMenu := True;
          if not HasImgLstBitmap then
            begin
              TextRect := ARect;
              break;
            end
          else
            begin
              if FMenu.IsRightToLeft then
                TextRect.Right  := TextRect.Right + 5
              else
                TextRect.Left  := TextRect.Left - 5;
              break;
            end

        end;

  if FTopMenu then
    begin
      ACanvas.brush.color := FMenuBarColor;
      ACanvas.Pen.Color := FMenuBarColor;
      ACanvas.FillRect(ARect);
      if FMenuItem.GetParentMenu.Items[FMenuItem.GetParentMenu.Items.Count-1] =
         FMenuItem then
        begin
          if FMenu.IsRightToLeft then
            ACanvas.Rectangle(3, ARect.Top, ARect.Right, ARect.Bottom)
          else
            ACanvas.Rectangle(ARect.Left, ARect.Top, FForm.ClientWidth+5, ARect.Bottom);
        end
      else
        if FMenu.IsRightToLeft then
          ACanvas.Rectangle(ARect.Left, ARect.Top, ARect.Right+7, ARect.Bottom);
    end
  else
    begin
      ACanvas.brush.color := FColor;
      ACanvas.FillRect(ARect);
      ACanvas.brush.color := FIconBackColor;
      ACanvas.FillRect(IconRect);
    end;


  if FMenuItem.Enabled then
    ACanvas.Font.Color := FFont.Color
  else
    ACanvas.Font.Color := FDisabledColor;

  if Selected and FDrawSelect then
    begin
      ACanvas.brush.Style := bsSolid;
      if FTopMenu then
        begin
          if FMenu.IsRightToLeft then
            begin
              X1 := ARect.Left + 6;
              X2 := ARect.Right + 7
            end
          else
            begin
              X1 := ARect.Left;
              X2 := ARect.Right;
            end;

          ACanvas.Pen.Color := $00606060;

          ACanvas.MoveTo (X1, ARect.Bottom-1);
          ACanvas.LineTo (X1, ARect.Top);
          ACanvas.LineTo (X2 - 8, ARect.Top);
          ACanvas.LineTo (X2 - 8, ARect.Bottom);
         //---------

          HoldColor:= GetShadeColor(FMenuBarColor, 10);
          ACanvas.Brush.Style := bsSolid;
          ACanvas.Brush.Color := HoldColor;
          ACanvas.Pen.Color := HoldColor;
          ACanvas.Rectangle(X2 - 7, ARect.Top + 1, X2 ,ARect.Bottom);

          HoldColor:= GetShadeColor(FMenuBarColor, 30);
          ACanvas.Brush.Color := HoldColor;
          ACanvas.Pen.Color := HoldColor;
          ACanvas.Rectangle(X2 - 7, ARect.Top + 3, X2 - 2 ,ARect.Bottom);

          HoldColor:= GetShadeColor(FMenuBarColor, 40);
          ACanvas.Brush.Color := HoldColor;
          ACanvas.Pen.Color := HoldColor;
          ACanvas.Rectangle(X2 - 7, ARect.Top + 5, X2 - 3 ,ARect.Bottom);

          HoldColor:= GetShadeColor(FMenuBarColor, 60);
          ACanvas.Brush.Color := HoldColor;
          ACanvas.Pen.Color := HoldColor;
          ACanvas.Rectangle(X2 - 7, ARect.Top + 6, X2 - 5 ,ARect.Bottom);

          //---

          ACanvas.Pen.Color :=FMenuBarColor;
          ACanvas.MoveTo(X2 - 5, ARect.Top + 1);
          ACanvas.LineTo(X2 - 1, ARect.Top + 1);
          ACanvas.LineTo(X2 - 1, ARect.Top + 6);

          ACanvas.MoveTo(X2 - 3, ARect.Top + 2);
          ACanvas.LineTo(X2 - 2, ARect.Top + 2);
          ACanvas.LineTo(X2 - 2, ARect.Top + 3);
          ACanvas.LineTo(X2 - 3, ARect.Top + 3);



          ACanvas.Pen.Color := GetShadeColor(FMenuBarColor, 10);
          ACanvas.MoveTo(X2 - 6, ARect.Top + 3);
          ACanvas.LineTo(X2 - 3, ARect.Top + 3);
          ACanvas.LineTo(X2 - 3, ARect.Top + 6);
          ACanvas.LineTo(X2 - 4, ARect.Top + 6);
          ACanvas.LineTo(X2 - 4, ARect.Top + 3);

          ACanvas.Pen.Color := GetShadeColor(FMenuBarColor, 30);
          ACanvas.MoveTo(X2 - 5, ARect.Top + 5);
          ACanvas.LineTo(X2 - 4, ARect.Top + 5);
          ACanvas.LineTo(X2 - 4, ARect.Top + 9);

          ACanvas.Pen.Color := GetShadeColor(FMenuBarColor, 40);
          ACanvas.MoveTo(X2 - 6, ARect.Top + 5);
          ACanvas.LineTo(X2 - 6, ARect.Top + 7);


        end
      else
        if FMenuItem.Enabled then
          begin
            ACanvas.brush.color := FSelectColor;
            ACanvas.FillRect(ARect);
            ACanvas.Pen.color := SelecBorderColor;
            ACanvas.Brush.Style := bsClear;
            ACanvas.RoundRect(Arect.Left, Arect.top, Arect.Right,
                              Arect.Bottom, 3, 3);

          end;
      DefColor := FSelectFontColor;
    end;

  if FMenuItem.RadioItem then
    begin
      if FMenuItem.Checked then
        begin
          ACanvas.Pen.color := FCheckedColor;
          ACanvas.Brush.Color := FColor;
          ACanvas.Brush.Style := bsSolid;
          if HasImgLstBitmap then
            begin
              ACanvas.RoundRect(CheckedRect.Left, CheckedRect.Top,
                                CheckedRect.Right, CheckedRect.Bottom,
                                6,6);
            end
          else
            begin
              ACanvas.Ellipse(CheckedRect);
            end;
        end;
    end
  else
    begin
      if (FMenuItem.Checked) then
        if (not HasImgLstBitmap) then
          begin
            ACanvas.Pen.color := FCheckedColor;
            ACanvas.Brush.Color := FColor;
            ACanvas.Brush.Style := bsSolid;
            ACanvas.Rectangle(CheckedRect);
            x1:= CheckedRect.Left + 1;
            x2 := CheckedRect.Top + 5;
            ACanvas.MoveTo(x1, x2);

            x1 := CheckedRect.Left + 4;
            x2 := CheckedRect.Bottom - 2;
            ACanvas.LineTo(x1, x2);
           //--
            x1:= CheckedRect.Left + 2;
            x2 := CheckedRect.Top + 5;
            ACanvas.MoveTo(x1, x2);

            x1 := CheckedRect.Left + 4;
            x2 := CheckedRect.Bottom - 3;
            ACanvas.LineTo(x1, x2);
           //--
            x1:= CheckedRect.Left + 2;
            x2 := CheckedRect.Top + 4;
            ACanvas.MoveTo(x1, x2);

            x1 := CheckedRect.Left + 5;
            x2 := CheckedRect.Bottom - 3;
            ACanvas.LineTo(x1, x2);
           //-----------------

            x1 := CheckedRect.Left + 4;
            x2 := CheckedRect.Bottom - 3;
            ACanvas.MoveTo(x1, x2);

            x1:= CheckedRect.Right + 2;
            x2 := CheckedRect.Top - 1;
            ACanvas.LineTo(x1, x2);
           //--
            x1 := CheckedRect.Left + 4;
            x2 := CheckedRect.Bottom - 2;
            ACanvas.MoveTo(x1, x2);

            x1:= CheckedRect.Right - 2;
            x2 := CheckedRect.Top + 3;
            ACanvas.LineTo(x1, x2);

          end
      else
        begin
          ACanvas.Pen.color := FCheckedColor;
          ACanvas.Brush.Color := FColor;
          ACanvas.Brush.Style := bsSolid;
          ACanvas.Rectangle(CheckedRect);
        end;
    end;

  if B <> nil then
    begin
      X1 := IConRect.Left ;
      if FIconWidth >= B.Width then
        X1 := X1 + ((FIconWidth  - B.Width ) div 2)
      else
        begin
          if FMenu.IsRightToLeft then
            X1 := IConRect.Right - b.Width - 2
          else
            X1 := X1 + 2;
        end;
      X2 := IConRect.top +  ((IConRect.Bottom - IConRect.Top) - B.Height) div 2;
      if (Selected) and (not FTopMenu) and (FMenuItem.Enabled)then
        begin
          dec(X1, 1);
          dec(X2, 1);
        end;
      ACanvas.Draw(X1, X2, B);
    end;


  if FMenuItem.Enabled then
    DefColor := FFont.Color;

  if Selected then
    DefColor := FSelectFontColor;
  if not FMenuItem.Enabled then
    DefColor := FDisabledColor;

  ACanvas.Font.color := DefColor;

  if not FMenuItem.IsLine then
    begin

      TextRect.Top := TextRect.Top +
        ((TextRect.Bottom - TextRect.Top) - ACanvas.TextHeight ('W')) div 2;

      SetBkMode(ACanvas.Handle, TRANSPARENT);


      if FMenu.IsRightToLeft then
        begin
          TextFormat := DT_RIGHT + DT_RTLREADING;
          Dec(TextRect.Right, 5);
        end
      else
        begin
          TextFormat := 0;
          Inc(TextRect.Left, 5);
        end;


      if FMenuItem.Default and FMenuItem.Enabled then
        begin

          Inc(TextRect.Top, 2);
          Inc(TextRect.Left, 1);
          Inc(TextRect.Right, 1);

          ACanvas.Font.color := clSilver;
          DrawtextEx(ACanvas.Handle,
                     PChar(txt),
                     Length(txt),
                     TextRect, TextFormat, nil);


          Dec(TextRect.Top, 1);
          Dec(TextRect.Left, 1);
          Dec(TextRect.Right, 1);

          ACanvas.Font.color := clSilver;
          DrawtextEx(ACanvas.Handle,
                     PChar(txt),
                     Length(txt),
                     TextRect, TextFormat, nil);


          Inc(TextRect.Left, 1);
          Inc(TextRect.Right, 1);

          ACanvas.Font.color := clGray;
          DrawtextEx(ACanvas.Handle,
                     PChar(txt),
                     Length(txt),
                     TextRect, TextFormat, nil);

          Dec(TextRect.Left, 1);
          Dec(TextRect.Right, 1);
          Dec(TextRect.Top, 1);

          ACanvas.Font.color := DefColor;
        end;


      DrawtextEx(ACanvas.Handle,
                 PChar(txt),
                 Length(txt),
                 TextRect, TextFormat, nil);



      txt := ShortCutToText(FMenuItem.ShortCut) +  ' ';

      if FMenu.IsRightToLeft then
        TextFormat := DT_LEFT
      else
        TextFormat := DT_RIGHT;

      DrawtextEx(ACanvas.Handle,
                 PChar(txt),
                 Length(txt),
                 TextRect, TextFormat, nil);

    end
  else
    begin
      if FMenu.IsRightToLeft then
        begin
          X1 := TextRect.Left;
          X2 := TextRect.Right - 7;
        end
      else
        begin
          X1 := TextRect.Left + 7;
          X2 := TextRect.Right;
        end;

      ACanvas.Pen.Color := FSeparatorColor;
      ACanvas.MoveTo(X1,
                     TextRect.Top +
                     Round((TextRect.Bottom - TextRect.Top) / 2));
      ACanvas.LineTo(X2 ,
                     TextRect.Top +
                     Round((TextRect.Bottom - TextRect.Top) / 2))
    end;

  B.free;

end;


procedure TXPMenu.SetActive(const Value: boolean);
begin

  FActive := Value;

  if FActive then
    begin
     InitMenueItems(false);
     InitMenueItems(true);
    end
  else
    InitMenueItems(false);
end;

procedure TXPMenu.SetForm(const Value: TForm);
var
  Hold: boolean;
begin
  if Value <> FForm then
    begin
      Hold := Active;
      Active := false;
      FForm := Value;
      if Hold then
        Active := True;
    end;
end;

procedure TXPMenu.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
end;

procedure TXPMenu.SetColor(const Value: TColor);
begin
  FColor := Value;
end;

procedure TXPMenu.SetIconBackColor(const Value: TColor);
begin
  FIconBackColor := Value;
end;

procedure TXPMenu.SetMenuBarColor(const Value: TColor);
begin
  FMenuBarColor := Value;
end;

procedure TXPMenu.SetCheckedColor(const Value: TColor);
begin
  FCheckedColor := Value;
end;

procedure TXPMenu.SetSeparatorColor(const Value: TColor);
begin
  FSeparatorColor := Value;
end;

procedure TXPMenu.SetSelecBorderColor(const Value: TColor);
begin
  FSelecBorderColor := Value;
end;

procedure TXPMenu.SetSelectColor(const Value: TColor);
begin
  FSelectColor := Value;
end;

procedure TXPMenu.SetDisabledColor(const Value: TColor);
begin
  FDisabledColor := Value;
end;

procedure TXPMenu.SetSelectFontColor(const Value: TColor);
begin
  FSelectFontColor := Value;
end;

procedure TXPMenu.SetIconWidth(const Value: integer);
begin
  FIconWidth := Value;
end;

procedure TXPMenu.SetDrawSelect(const Value: boolean);
begin
  FDrawSelect := Value;
end;

function GetShadeColor(clr:TColor; Value: byte): TColor;
var
  r,g,b: byte;

begin
  r := Clr and $000000ff;
  g := (Clr and $0000ff00)  shr 8;
  b := (Clr and $00ff0000)  shr 16;
  r := abs(r - value);
  g := abs(g - value);
  b := abs(b - value);

  Result := TColor(RGB(r, g, b));
end;

end.
